# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


RES_PATH = ''

class Global(object):
    """Current package global parameters"""

    COLON = ":"
    FILE_READ = "rb"
    FILE_WRITE = "wb"
    TAB = "\t"
    COMMA = ","
    TIME_FORM = "%Y-%m-%d %H:%M:%S"
    global RES_PATH