# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

class PredictConstance(object):
    """Constance of predict class."""

    BASE_CONFIG = "base_config"
    MODEL_NAME = "model_name"
    MODEL_CONFIG = "model_config"

    CV_CONFIG = "cv_config"

    FEATURE_ENGINEERING_CONFIG = "feature_engineering_config"

    ANALYSIS_CONFIG = "analysis_config"

    DATA_SPLITE_CONFIG = "data_split_config"

    RESULT_PATH = "result_path"
    PREDICT_FILE = "predict_result.txt"
    TEST_SCORE = "test_score.txt"

    PRE = "pre"
    PROBA = "proba"
